'use strict';

const { GObject, Gtk, Gio } = imports.gi;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const BuilderScope = GObject.registerClass({
    Implements: [Gtk.BuilderScope],
}, class BuilderScope extends GObject.Object {
    _init() {
        super._init();
    }

    vfunc_create_closure(builder, handlerName, flags, connectObject) {
        if (flags & Gtk.BuilderClosureFlags.SWAPPED)
            throw new Error('Unsupported template signal flag "swapped"');

        if (typeof this[handlerName] === 'undefined')
            throw new Error(`${handlerName} is undefined`);

        return this[handlerName].bind(connectObject || this);
    }
});

function init() {

}

function buildPrefsWidget() {
    let builder = new Gtk.Builder();

    builder.set_scope(new BuilderScope());
    let UIFilePath = Me.dir.get_child("ui").get_path() + '/prefs.ui';
    builder.add_from_file(UIFilePath);

    let obj = builder.get_object('main_prefs');

    let settings = ExtensionUtils.getSettings(Me.metadata['schema-id']);
    let touchpad_speed = builder.get_object('touchpad-speed-scale');
    touchpad_speed.set_value(settings.get_double('touchpad-speed-scale'));
    settings.bind('touchpad-speed-scale', touchpad_speed.adjustment, 'value', Gio.SettingsBindFlags.DEFAULT);

    let alttab_delay = builder.get_object('alttab-delay');
    alttab_delay.set_value(settings.get_int('alttab-delay'));
    settings.bind('alttab-delay', alttab_delay.adjustment, 'value', Gio.SettingsBindFlags.DEFAULT);

    return obj;
}